@@ -4,6 +4,7 @@
 #ifdef DLIB_RAND_KERNEl_ABSTRACT_
 
 #include <string>
+#include <complex>
 #include "../uintn.h"
 
 namespace dlib
@@ -157,8 +158,9 @@ namespace dlib
                 requires
                     - begin <= end
                 ensures
-                    - returns a random integer selected from the range: begin <= N < end
-                      The integer is selected uniformly at random.
+                    - returns a random integer N selected from the range: begin <= N < end
+                      The integer is selected uniformly at random.  If begin==end then
+                      begin is returned.
             !*/
 
             long long get_integer(
@@ -179,6 +181,48 @@ namespace dlib
                       with mean 0 and standard deviation 1. 
             !*/
 
+            std::complex<double> get_random_complex_gaussian (
+            );
+            /*!
+                ensures
+                    - returns a random complex number sampled from a Gaussian distribution 
+                      with mean 0 and standard deviation 1. 
+            !*/
+            
+            double get_random_exponential (
+                double lambda
+            );
+            /*!
+                ensures
+                    - returns a random number sampled from an exponential distribution
+                      with rate parameter lambda
+            !*/
+
+            double get_random_weibull (
+                double lambda,
+                double k,
+                double gamma
+            );
+            /*!
+                ensures
+                    - returns a random number sampled from a Weibull distribution
+                      with shape parameter k, scale parameter lambda and 
+                      threshold parameter gamma.
+            !*/
+
+            double get_random_beta (
+                double alpha,
+                double beta,
+            )
+            /*!
+                requires
+                    - alpha > 0
+                    - beta > 0
+                ensures
+                    - returns a random number sampled from a Beta distribution
+                      with shape parameters alpha and beta.
+            !*/
+
             void swap (
                 rand& item
             );
