@@ -21,7 +21,7 @@
 #include <dlib/dir_nav.h>
 
 
-const char* VERSION = "1.13";
+const char* VERSION = "1.19";
 
 
 
@@ -60,8 +60,8 @@ void create_new_dataset (
         {
             // then parser[i] should be a directory
 
-            std::vector<file> files = get_files_in_directory_tree(parser[i], 
-                                                                  match_endings(".png .PNG .jpeg .JPEG .jpg .JPG .bmp .BMP .dng .DNG .gif .GIF"),
+            std::vector<file> files = get_files_in_directory_tree(parser[i],
+                                                                  match_endings(".png .PNG .jpeg .JPEG .jpg .JPG .bmp .BMP .dng .DNG .gif .GIF .webp .WEBP"),
                                                                   depth);
             sort(files.begin(), files.end());
 
@@ -127,6 +127,44 @@ int split_dataset (
 
 // ----------------------------------------------------------------------------------------
 
+int make_train_test_splits (
+    const command_line_parser& parser
+)
+{
+    if (parser.number_of_arguments() != 1)
+    {
+        cerr << "The --split-train-test option requires you to give one XML file on the command line." << endl;
+        return EXIT_FAILURE;
+    }
+
+    const double train_frac = get_option(parser, "split-train-test", 0.5);
+
+    dlib::image_dataset_metadata::dataset data, data_train, data_test;
+    load_image_dataset_metadata(data, parser[0]);
+
+    data_train.name = data.name;
+    data_train.comment = data.comment;
+    data_test.name = data.name;
+    data_test.comment = data.comment;
+
+    const unsigned long num_train_images = static_cast<unsigned long>(std::round(train_frac*data.images.size()));
+
+    for (unsigned long i = 0; i < data.images.size(); ++i)
+    {
+        if (i < num_train_images)
+            data_train.images.push_back(data.images[i]);
+        else
+            data_test.images.push_back(data.images[i]);
+    }
+
+    save_image_dataset_metadata(data_train, left_substr(parser[0],".") + "_train.xml");
+    save_image_dataset_metadata(data_test, left_substr(parser[0],".") + "_test.xml");
+
+    return EXIT_SUCCESS;
+}
+
+// ----------------------------------------------------------------------------------------
+
 void print_all_labels (
     const dlib::image_dataset_metadata::dataset& data
 )
@@ -294,35 +332,33 @@ void rotate_dataset(const command_line_parser& parser)
 
         load_image(img, metadata.images[i].filename);
         const point_transform_affine tran = rotate_image(img, temp, angle*pi/180);
+        metadata.images[i].width = temp.nc();
+        metadata.images[i].height = temp.nr();
         if (parser.option("jpg"))
         {
             filename = to_jpg_name(filename);
             save_jpeg(temp, filename,JPEG_QUALITY);
         }
+#ifdef DLIB_WEBP_SUPPORT
+        else if (parser.option("webp"))
+        {
+            filename = to_webp_name(filename);
+            const float webp_quality = std::stof(parser.option("webp").argument());
+            save_webp(temp, filename, webp_quality);
+        }
+#endif
         else
         {
             save_png(temp, filename);
         }
 
+        rectangle_transform rtran = tran;
         for (unsigned long j = 0; j < metadata.images[i].boxes.size(); ++j)
         {
-            const rectangle rect = metadata.images[i].boxes[j].rect;
-            rectangle newrect;
-            newrect += tran(rect.tl_corner());
-            newrect += tran(rect.tr_corner());
-            newrect += tran(rect.bl_corner());
-            newrect += tran(rect.br_corner());
-            // now make newrect have the same area as the starting rect.
-            double ratio = std::sqrt(rect.area()/(double)newrect.area());
-            newrect = centered_rect(newrect, newrect.width()*ratio, newrect.height()*ratio);
-            metadata.images[i].boxes[j].rect = newrect;
+            metadata.images[i].boxes[j].rect = rtran(metadata.images[i].boxes[j].rect);
 
-            // rotate all the object parts
-            std::map<std::string,point>::iterator k;
-            for (k = metadata.images[i].boxes[j].parts.begin(); k != metadata.images[i].boxes[j].parts.end(); ++k)
-            {
-                k->second = tran(k->second); 
-            }
+            for (auto& p : metadata.images[i].boxes[j].parts)
+                p.second = tran(p.second);
         }
 
         metadata.images[i].filename = filename;
@@ -333,6 +369,32 @@ void rotate_dataset(const command_line_parser& parser)
 
 // ----------------------------------------------------------------------------------------
 
+void add_width_and_height_metadata(const command_line_parser& parser)
+{
+    for (unsigned long i = 0; i < parser.number_of_arguments(); ++i) {
+        image_dataset_metadata::dataset metadata;
+        const string datasource = parser[i];
+        load_image_dataset_metadata(metadata,datasource);
+
+        // Set the current directory to be the one that contains the
+        // metadata file. We do this because the file might contain
+        // file paths which are relative to this folder.
+        set_current_dir(get_parent_directory(file(datasource)));
+
+        parallel_for(0, metadata.images.size(), [&](long i) 
+        {
+            array2d<rgb_pixel> img;
+            load_image(img, metadata.images[i].filename);
+            metadata.images[i].width = img.nc();
+            metadata.images[i].height = img.nr();
+        });
+
+        save_image_dataset_metadata(metadata, datasource);
+    }
+}
+
+// ----------------------------------------------------------------------------------------
+
 int resample_dataset(const command_line_parser& parser)
 {
     if (parser.number_of_arguments() != 1)
@@ -421,12 +483,22 @@ int resample_dataset(const command_line_parser& parser)
             std::ostringstream sout;
             sout << hex << murmur_hash3_128bit(&chip[0][0], chip.size()*sizeof(chip[0][0])).second;
             dimg.filename = data.images[i].filename + "_RESAMPLED_"+sout.str()+".png";
+            dimg.width = chip.nc();
+            dimg.height = chip.nr();
 
             if (parser.option("jpg"))
             {
                 dimg.filename = to_jpg_name(dimg.filename);
                 save_jpeg(chip,dimg.filename, JPEG_QUALITY);
             }
+#ifdef DLIB_WEBP_SUPPORT
+            else if (parser.option("webp"))
+            {
+                dimg.filename = to_webp_name(dimg.filename);
+                const float webp_quality = std::stof(parser.option("webp").argument());
+                save_webp(chip, dimg.filename, webp_quality);
+            }
+#endif
             else
             {
                 save_png(chip,dimg.filename);
@@ -452,9 +524,9 @@ int tile_dataset(const command_line_parser& parser)
 
     string out_image = parser.option("tile").argument();
     string ext = right_substr(out_image,".");
-    if (ext != "png" && ext != "jpg")
+    if (ext != "png" && ext != "jpg" && ext != "webp")
     {
-        cerr << "The output image file must have either .png or .jpg extension." << endl;
+        cerr << "The output image file must have either .png, .jpg or .webp extension." << endl;
         return EXIT_FAILURE;
     }
 
@@ -496,9 +568,21 @@ int tile_dataset(const command_line_parser& parser)
     chdir.revert();
 
     if (ext == "png")
+    {
         save_png(tile_images(images), out_image);
+    }
+#ifdef DLIB_WEBP_SUPPORT
+    else if (ext == "webp")
+    {
+        // Lossless by default
+        const float webp_quality = get_option(parser, "webp", 101.f);
+        save_webp(tile_images(images), out_image, webp_quality);
+    }
+#endif
     else
-        save_jpeg(tile_images(images), out_image);
+    {
+        save_jpeg(tile_images(images), out_image, JPEG_QUALITY);
+    }
 
     return EXIT_SUCCESS;
 }
@@ -539,12 +623,17 @@ int main(int argc, char** argv)
                                     "the md5 hash of each image file and removing duplicate images. " );
         parser.add_option("rmdiff","Set the ignored flag to true for boxes marked as difficult.");
         parser.add_option("rmtrunc","Set the ignored flag to true for boxes that are partially outside the image.");
+        parser.add_option("box-images","Add a box to each image that contains the entire image.");
         parser.add_option("sort-num-objects","Sort the images listed an XML file so images with many objects are listed first.");
         parser.add_option("sort","Alphabetically sort the images in an XML file.");
         parser.add_option("shuffle","Randomly shuffle the order of the images listed in an XML file.");
         parser.add_option("seed", "When using --shuffle, set the random seed to the string <arg>.",1);
         parser.add_option("split", "Split the contents of an XML file into two separate files.  One containing the "
             "images with objects labeled <arg> and another file with all the other images. ",1);
+        parser.add_option("split-train-test", "Split the contents of an XML file into two separate files.  A training "
+            "file containing <arg> fraction of the images and a testing file containing the remaining (1-<arg>) images. "
+            "The partitioning is done deterministically by putting the first images in the input xml file into the training split "
+            "and the later images into the test split.",1);
         parser.add_option("add", "Add the image metadata from <arg1> into <arg2>.  If any of the image "
                                  "tags are in both files then the ones in <arg2> are deleted and replaced with the "
                                  "image tags from <arg1>.  The results are saved into merged.xml and neither <arg1> or "
@@ -557,7 +646,9 @@ int main(int argc, char** argv)
                                         "The parts are instead simply mirrored to the flipped dataset.", 1);
         parser.add_option("rotate", "Read an XML image dataset and output a copy that is rotated counter clockwise by <arg> degrees. "
                                   "The output is saved to an XML file prefixed with rotated_<arg>.",1);
-        parser.add_option("cluster", "Cluster all the objects in an XML file into <arg> different clusters and save "
+        parser.add_option("add-width-height-metadata", "Open the given xml files and set the width and height image metadata fields "
+                            "for every image. This involves loading each image to find these values.");
+        parser.add_option("cluster", "Cluster all the objects in an XML file into <arg> different clusters (pass 0 to find automatically) and save "
                                      "the results as cluster_###.xml and cluster_###.jpg files.",1);
         parser.add_option("ignore", "Mark boxes labeled as <arg> as ignored.  The resulting XML file is output as a separate file and the original is not modified.",1);
         parser.add_option("rmlabel","Remove all boxes labeled <arg> and save the results to a new XML file.",1);
@@ -565,6 +656,9 @@ int main(int argc, char** argv)
         parser.add_option("rmignore","Remove all boxes marked ignore and save the results to a new XML file.");
         parser.add_option("rm-if-overlaps","Remove all boxes labeled <arg> if they overlap any box not labeled <arg> and save the results to a new XML file.",1);
         parser.add_option("jpg", "When saving images to disk, write them as jpg files instead of png.");
+#ifdef DLIB_WEBP_SUPPORT
+        parser.add_option("webp", "When saving images to disk, write them as webp files instead of png or jpg, using <arg> as the quality factor.", 1);
+#endif
 
         parser.set_group_name("Cropping sub images");
         parser.add_option("resample", "Crop out images that are centered on each object in the dataset. "
@@ -581,7 +675,7 @@ int main(int argc, char** argv)
         const char* singles[] = {"h","c","r","l","files","convert","parts","rmdiff", "rmtrunc", "rmdupes", "seed", "shuffle", "split", "add", 
                                  "flip-basic", "flip", "rotate", "tile", "size", "cluster", "resample", "min-object-size", "rmempty",
                                  "crop-size", "cropped-object-size", "rmlabel", "rm-other-labels", "rm-if-overlaps", "sort-num-objects", 
-                                 "one-object-per-image", "jpg", "rmignore", "sort"};
+                                 "one-object-per-image", "jpg", "rmignore", "sort", "split-train-test", "box-images", "add-width-height-metadata"};
         parser.check_one_time_options(singles);
         const char* c_sub_ops[] = {"r", "convert"};
         parser.check_sub_options("c", c_sub_ops);
@@ -600,11 +694,13 @@ int main(int argc, char** argv)
         parser.check_incompatible_options("c", "rm-if-overlaps");
         parser.check_incompatible_options("c", "rmdupes");
         parser.check_incompatible_options("c", "rmtrunc");
+        parser.check_incompatible_options("c", "box-images");
         parser.check_incompatible_options("c", "add");
         parser.check_incompatible_options("c", "flip");
         parser.check_incompatible_options("c", "flip-basic");
         parser.check_incompatible_options("flip", "flip-basic");
         parser.check_incompatible_options("c", "rotate");
+        parser.check_incompatible_options("c", "add-width-height-metadata");
         parser.check_incompatible_options("c", "rename");
         parser.check_incompatible_options("c", "ignore");
         parser.check_incompatible_options("c", "parts");
@@ -618,6 +714,7 @@ int main(int argc, char** argv)
         parser.check_incompatible_options("l", "flip");
         parser.check_incompatible_options("l", "flip-basic");
         parser.check_incompatible_options("l", "rotate");
+        parser.check_incompatible_options("l", "add-width-height-metadata");
         parser.check_incompatible_options("files", "rename");
         parser.check_incompatible_options("files", "ignore");
         parser.check_incompatible_options("files", "add");
@@ -625,22 +722,27 @@ int main(int argc, char** argv)
         parser.check_incompatible_options("files", "flip");
         parser.check_incompatible_options("files", "flip-basic");
         parser.check_incompatible_options("files", "rotate");
+        parser.check_incompatible_options("files", "add-width-height-metadata");
         parser.check_incompatible_options("add", "flip");
         parser.check_incompatible_options("add", "flip-basic");
         parser.check_incompatible_options("add", "rotate");
+        parser.check_incompatible_options("add", "add-width-height-metadata");
         parser.check_incompatible_options("add", "tile");
         parser.check_incompatible_options("flip", "tile");
         parser.check_incompatible_options("flip-basic", "tile");
         parser.check_incompatible_options("rotate", "tile");
+        parser.check_incompatible_options("add-width-height-metadata", "tile");
         parser.check_incompatible_options("cluster", "tile");
         parser.check_incompatible_options("resample", "tile");
         parser.check_incompatible_options("flip", "cluster");
         parser.check_incompatible_options("flip-basic", "cluster");
         parser.check_incompatible_options("rotate", "cluster");
+        parser.check_incompatible_options("add-width-height-metadata", "cluster");
         parser.check_incompatible_options("add", "cluster");
         parser.check_incompatible_options("flip", "resample");
         parser.check_incompatible_options("flip-basic", "resample");
         parser.check_incompatible_options("rotate", "resample");
+        parser.check_incompatible_options("add-width-height-metadata", "resample");
         parser.check_incompatible_options("add", "resample");
         parser.check_incompatible_options("shuffle", "tile");
         parser.check_incompatible_options("sort-num-objects", "tile");
@@ -668,14 +770,23 @@ int main(int argc, char** argv)
         parser.check_incompatible_options("rmdupes", "ignore");
         parser.check_incompatible_options("rmtrunc", "rename");
         parser.check_incompatible_options("rmtrunc", "ignore");
+        parser.check_incompatible_options("box-images", "rename");
+        parser.check_incompatible_options("box-images", "ignore");
+#ifdef DLIB_WEBP_SUPPORT
+        parser.check_incompatible_options("jpg", "webp");
+#endif
         const char* convert_args[] = {"pascal-xml","pascal-v1","idl"};
         parser.check_option_arg_range("convert", convert_args);
-        parser.check_option_arg_range("cluster", 2, 999);
+        parser.check_option_arg_range("cluster", 0, 999);
         parser.check_option_arg_range("rotate", -360, 360);
         parser.check_option_arg_range("size", 10*10, 1000*1000);
         parser.check_option_arg_range("min-object-size", 1, 10000*10000);
         parser.check_option_arg_range("cropped-object-size", 4, 10000*10000);
         parser.check_option_arg_range("crop-size", 1.0, 100.0);
+        parser.check_option_arg_range("split-train-test", 0.0, 1.0);
+#ifdef DLIB_WEBP_SUPPORT
+        parser.check_option_arg_range("webp", 0.f, std::numeric_limits<float>::max());
+#endif
 
         if (parser.option("h"))
         {
@@ -703,6 +814,12 @@ int main(int argc, char** argv)
             return EXIT_SUCCESS;
         }
 
+        if (parser.option("add-width-height-metadata"))
+        {
+            add_width_and_height_metadata(parser);
+            return EXIT_SUCCESS;
+        }
+
         if (parser.option("v"))
         {
             cout << "imglab v" << VERSION 
@@ -954,6 +1071,29 @@ int main(int argc, char** argv)
             return EXIT_SUCCESS;
         }
 
+        if (parser.option("box-images"))
+        {
+            if (parser.number_of_arguments() != 1)
+            {
+                cerr << "The --box-images option requires you to give one XML file on the command line." << endl;
+                return EXIT_FAILURE;
+            }
+
+            dlib::image_dataset_metadata::dataset   data;
+            load_image_dataset_metadata(data, parser[0]);
+            {
+                locally_change_current_dir chdir(get_parent_directory(file(parser[0])));
+                parallel_for(0, data.images.size(), [&](long i) 
+                {
+                    array2d<unsigned char> img;
+                    load_image(img, data.images[i].filename);
+                    data.images[i].boxes.emplace_back(get_rect(img));
+                });
+            }
+            save_image_dataset_metadata(data, parser[0]+".boxed.xml");
+            return EXIT_SUCCESS;
+        }
+
         if (parser.option("rmtrunc"))
         {
             if (parser.number_of_arguments() != 1)
@@ -1016,6 +1156,11 @@ int main(int argc, char** argv)
             return split_dataset(parser);
         }
 
+        if (parser.option("split-train-test"))
+        {
+            return make_train_test_splits(parser);
+        }
+
         if (parser.option("shuffle"))
         {
             if (parser.number_of_arguments() != 1)
