@@ -224,7 +224,12 @@ void propagate_boxes(
 
     array2d<rgb_pixel> img1, img2;
     dlib::load_image(img1, data.images[prev].filename);
+    data.images[prev].width = img1.nc();
+    data.images[prev].height = img1.nr();
+
     dlib::load_image(img2, data.images[next].filename);
+    data.images[next].width = img2.nc();
+    data.images[next].height = img2.nr();
     for (unsigned long i = 0; i < data.images[prev].boxes.size(); ++i)
     {
         correlation_tracker tracker;
@@ -343,6 +348,16 @@ on_keydown (
             last_keyboard_jump_pos_update = 0;
         }
 
+        if (key == '=')
+        {
+            display.zoom_in();
+        }
+
+        if (key == '-')
+        {
+            display.zoom_out();
+        }
+
         if (key == 'd' && (state&base_window::KBD_MOD_ALT))
         {
             remove_selected_images();
@@ -503,6 +518,8 @@ load_image(
     try
     {
         dlib::load_image(img, metadata.images[idx].filename);
+        metadata.images[idx].width = img.nc();
+        metadata.images[idx].height = img.nr();
         set_title(metadata.name + " #"+cast_to_string(idx)+": " +metadata.images[idx].filename);
     }
     catch (exception& e)
@@ -533,6 +550,8 @@ load_image_and_set_size(
     try
     {
         dlib::load_image(img, metadata.images[idx].filename);
+        metadata.images[idx].width = img.nc();
+        metadata.images[idx].height = img.nr();
         set_title(metadata.name + " #"+cast_to_string(idx)+": " +metadata.images[idx].filename);
     }
     catch (exception& e)
@@ -644,7 +663,11 @@ display_about(
     sout << wrap_string("It is also possible to label object parts by selecting a rectangle and "
                         "then right clicking.  A popup menu will appear and you can select a part label. "
                         "Note that you must define the allowable part labels by giving --parts on the "
-                        "command line.  An example would be '--parts \"leye reye nose mouth\"'."
+                        "command line.  An example would be '--parts \"leye reye nose mouth\"'. "
+                        "Alternatively, if you don't give --parts you can simply select a rectangle and shift+left "
+                        "click to add parts. Parts added this way will be labeled with integer labels starting from 0. "
+                        "You can only use this simpler part adding mode if all the parts in a rectangle are already "
+                        "labeled with integer labels or the rectangle has no parts at all."
                         ,0,0) << endl << endl;
 
     sout << wrap_string("Press the down or s key to select the next image in the list and the up or w "
